/*
--------------------------------------------------------------------------------
cfr_nc.do
by Jesse Rothstein

This is the master do-file of the "cfr_nc" project, which replicates
Chetty-Friedman-Rockoff's (2014a,b) analysis using data from North Carolina.

See "README.txt" for instructions on how to run this project

The principal investigator for this project is:

	Jesse Rothstein    rothstein@berkeley.edu
	
I am grateful to Robert Picard, picard@netbox.com, for the "project.ado"
program used to implement the project.

--------------------------------------------------------------------------------
Modified, JR, 3/23/15: 
   Add code to confirm existence of scratch directories.
   Separate cfr_analysis_final_7.do into 7A, 7B -- the latter reproduces CFRs Table 6.
   Modify baseline control vector to include some missing value indicators previously omitted.

Modified, JR, 1/18/17: 
   Add README.txt file
   Change to run under Stata 14
   
*/


**************************** PRELIMINARIES *************************************

	version 12.1

* Common settings

	set more off
	set varabbrev off	// for long projects, it's best not to abbreviate
	set linesize 132	// use 7pt font for printing
	
	*Describe Stata version.
	 version
	 creturn list
   set
   assert "`c(version)'"=="12.1"
   assert "`c(stata_version)'"=="14"
	
  which project
  *Should be:
    */accounts/projects/nceduc/ado/plus/p/project.ado
    *! version 1.3.1  22dec2013  picard@netbox.com
  *To check for updates, type:
  * adoupdate project

  *Confirm that various directories exist, before we get going
   project, doinfo
	 local pdir "`r(pdir)'"						  	    // the project's main dir.
	 local dofile "`r(dofile)'"						    // do-file's stub name
   local sig {bind:{hi:[RP : `dofile'.do]}}	// a signature in notes   
   *NOTE: NEED TO PICK A DIRECTORY THAT HAS SPACE FOR MANY GB OF FILES.
   *ALSO, THIS IS HARD CODED INTO EACH PROGRAM -- NEED TO MAKE SURE THESE ARE UP TO DATE.
   *ON MY SYSTEM, I ALSO NEED TO SET THE TMPDIR TO REDIRECT STATA TEMP FILES,
   *  AS WITHOUT THIS THE PROGRAM OVER-FILLS THE TEMP DRIVE AND CRASHES.
   *  I DO THIS WITH THE FOLLOWING TWO COMMANDS, BEFORE EXECUTING STATA:
   *    STATATMP="/var/tmp/scratch/nceduc/jesse"
   *    export STATATMP
   
   local scratchdir "scratch"
   confirm new file `scratchdir'/test.dta
   confirm new file `scratchdir'/ncdata/test.dta
   confirm new file `scratchdir'/cfrdata/test.dta
   confirm new file `scratchdir'/cfrdata/tfx/test.dta
   confirm new file `scratchdir'/cfrdata/tfx/table6/test.dta
   confirm new file `scratchdir'/cfrdata/eventstudy/test.dta
   confirm new file `scratchdir'/replication/test.dta
   confirm new file `scratchdir'/extension/test.dta
   confirm new file `scratchdir'/longrun/test.dta
   
   
  * This directory includes a local copy of -project- in a zip archive. This
  * is done to ensure replicability in case of future changes in -project- or
  * if -project- becomes unavailable, or if you are unable to install it from
  * the SSC archives. Make it part of the project
 	project, relies_on("extras/project.zip")
 	project, relies_on("extras/estout.zip")

  * Make the read me file part of the project
 	project, relies_on("readme.txt")

**************************** DATA PREPARATION *************************************

* Preparatory -- get data into shape
   project, do("prepdata/prepdata_students.do")
   project, do("prepdata/prepdata_SAR.do")
   project, do("prepdata/validteachers.do")
   project, do("prepdata/prepdata_mbuild.do")
   project, do("prepdata/prepdata_longrun.do")

**************************** CFR PROGRAMS *************************************

*Plug these data into CFR programs to generate CFR-style samples;
  *CFR program 1: "create_schd_work_final.do"
    *Make a basic student-level dataset with all controls
      project, do("cfr/prepcfrsamp_1.do")
    *Define sample (still at student level) and sample counts   
      project, do("cfr/prepcfrsamp_2.do")  
    *Create various aggregated (school-grade, teacher-year, etc.) samples
      project, do("cfr/prepcfrsamp_3.do")  
    *Make file for long-run analysis (equivalent of CFR IRS files
      project, do("cfr/prepcfrsamp_4.do")  
  *CFR program 2: "Analysis_final.do"
    *Make teacher-class weights and school-grade-level means (CFR program lines 95-355)
     project, do("cfr/cfr_analysis_final_pt1.do")
    *Construct value-added estimates and forecasts  (CFR program lines 361-513)
     project, do("cfr/cfr_analysis_final_pt2.do")
    *Report autocovariance of VA at various lags (CFR-A, Table 2) (CFR program lines 517-599)
     project, do("cfr/cfr_analysis_final_pt3.do")
    *Create "cross-class" analysis sample (CFR program lines 601-812)
     project, do("cfr/cfr_analysis_final_pt4.do")
    *Summary statistics, observational regressions and bin scatter plots (CFR program lines 815-1200)
    *(CFR-A, Table 3 & Fig 2A)
     project, do("cfr/cfr_analysis_final_pt5.do")
    *Prepare and implement quasi-experimental analysis (CFR-A, Table 4, Fig 4A-B, & Fig 5B) (CFR program lines 1203-1393)
     project, do("cfr/cfr_analysis_final_pt6.do")

    *Quasi-experimental analysis robustness (CFR-A, Table 5) (CFR program lines 1396-1495)
     project, do("cfr/cfr_analysis_final_pt7A.do")
    *Quasi-experimental analysis robustness (CFR-A, Table 6) (CFR program lines 1498-1721)
     project, do("cfr/cfr_analysis_final_pt7B.do")

    *Association between student characteristics and teacher VA (CFR-A, Table 7 [now appendix table 2]) (CFR program lines 1724-1838)
     project, do("cfr/cfr_analysis_final_pt8.do")
    *Teacher switcher event studies, misc other (CFR-A, Figures 1A, 3, A-1A) (CFR program lines 1841-2671)
     project, do("cfr/cfr_analysis_final_pt9.do")
     
    *NOTE: I adapt much less of the second half of CFR's program, pertaining to
    *paper 2, as I do not have the same long-run outcome data so the changes
    *would be quite dramatic.  The "longrun" programs below conduct an exercise analogous to CFR's
    *analysis of long-run outcomes.    
    
    *Long-run "cross-class" analysis (CFR-B, Tables 1-2) (CFR program lines 2671-2821)
     project, do("cfr/cfr_analysis_final_pt10.do")
    *Long-run "quasi-experimental" analysis (CFR-B, Table 5) (CFR program lines 2987-3067)
     project, do("cfr/cfr_analysis_final_pt11.do")

**************************** REPLICATION **********************************************
	 project, do(replication/basicreplication.do)
   project, do(replication/cfr1_table5.do)
   project, do(replication/cfr1_fig1a.do)
   project, do(replication/summstats.do)

**************************** REPLICATION AND EXTENSION ********************************
   project, do(extension/demogpredict.do)
   project, do(extension/classlevel.do)
   project, do(extension/imputations.do)
   project, do(extension/preparequasi.do)
   *Analysis of different types of moves (between grades, between schools, etc.).
   *Not needed for paper, except that "movetypes.do" identifies followers used in apptab_leavenout.do
   project, do(extension/movetypes.do)
   project, do(extension/movetypes_quasi.do)
   project, do(extension/movetypes_analysis.do)

   *Reproduce basic specifications
   project, do(extension/table1.do)
   *Falsification test
   project, do(extension/table2.do)
   *Quasiexperiment with controls
   project, do(extension/table3.do)
   *Sorting by demographics
   project, do(extension/demogsorting.do)
   
   project, do(extension/vingtileplots.do)

   *Extra analyses, for appendix B
    *Robustness to excluding followers, leaving n out, etc.
   project, do(extension/apptab_leavenout.do)
   project, do(extension/apptab_altimpute.do)
   
**************************** LONG-RUN OUTCOMES *************************************
   project, do(longrun/makesamp_observational.do)
   project, do(longrun/makesamp_quasi.do)
   project, do(longrun/observationalregs.do)
   project, do(longrun/quasiregs.do)
   
**************************** EXTRAS *************************************
   project, do(replication/paperstatistics.do)
   project, do(extension/cfrresponse_sims_t2.do)


