****************
* This simulation shows that imputing zeros reduces the coefficient in the
*regression of changes in scores on changes in mean VA when VA is correlated
*across teachers in a cell
****************

****************
* cfrresponse_sims_t2.do
* Modified by Jesse Rothstein, 7/21/15.
* Based on CFR program included in their January 2015 reply to Rothstein's comment.
* Modifications: Program-ify it, allowing for (a) choice of ICC and (b) choice
*  of the number of years on which the VA prediction is based.
****************


version 13
clear *
matrix drop _all
estimates clear
set varabbrev off /*JR added*/

cap project, doinfo
if _rc==0 {
	local doasproject=1
	local pdir "`r(pdir)'"						  	    // the project's main dir.
	local dofile "`r(dofile)'"						    // do-file's stub name
  local sig {bind:{hi:[RP : `dofile'.do]}}	// a signature in notes

}
else {
  local doasproject=0
	local pdir "~/teachereffects/cfr_replic/project"
	local dofile "cfrresponse_sims_t2"
	local sig "Not run as part of project!"
	
}
local thisdir "`pdir'/extension"

clear *
estimates clear
_eststo clear


cap program drop onesim
program define onesim
syntax, [icc(real 0.35) eb nyears(real 1) reliability(real 0.4) modname(string) n(real 1000000)]

*CFR code, with slight modifications (indicated by comments)
****************
* This simulation shows that imputing zeros reduces the coefficient in the
*regression of changes in scores on changes in mean VA when VA is correlated
*across teachers in a cell
****************
drop _all
*1 Generate data at the school-year-teacher level
set obs `n'
set seed 509870
g year = mod(_n-1,2)+1
g teacher = ceil(_n/2)
g school = ceil(teacher/2)

*2 Generate correlated VA within cells
* CFR original code: global corr = 0.35
  global corr=`icc'
tsset teacher year
g va = rnormal(0,.1) if year == 1
replace va = (${corr}*l.va + sqrt(1-${corr}^2)*rnormal(0,.1)) if year == 2

*JR addition: Incorporate idea that the VA score is an EB prediction
 *Draw number for missingness first, so it is has the same seed as in CFRs simulation
  g rand = runiform()

 if "`eb'"~="" {
   gen va_obstot=0
   forvalues n=0/`nyears' {
     gen va_obs`n'=va + sqrt((1-`reliability')/(`reliability'))*rnormal(0, 0.1)
	 if `n'>0 replace va_obstot=va_obstot+va_obs`n'
   }
   gen vaavg=va_obstot/`nyears'
   reg va_obs0 vaavg
   predict va_meas, xb
 }
 else gen va_meas=va
	 
*3 Generate missing data
g miss = rand<.2
g va_miss = va_meas
replace va_miss = . if miss==1

*4 Generate scores
g score = va + rnormal(0,1)
g score_miss = score
replace score_miss = . if va_miss == .

*5 Imputation of 0 for missing data
g va_impute = va_miss
replace va_impute = 0 if va_miss == .

*6 Collapse to school-year and run regressions
collapse va va_miss va_impute va_meas score score_miss, by(school year)
tsset school year

*7 Results

*CFR regressions:
if "`modname'"=="" {
_eststo clear
_eststo Full_Sample: reg d.score d.va_meas // Coefficient in full sample (ideal data)
_eststo No_Missing: reg d.score_miss d.va_miss // Coefficient on subsample with no missing data

_eststo Impute_0s: reg d.score d.va_impute // Coefficient with imputation is downward-biased
_eststo Table1_Col5: reg d.score_miss d.va_impute // Rothstein Table 1, Col 5 spec: use scores for those with non-missing data on LHS and imputed VA on RHS
_eststo Table1_Col6: reg d.score d.va_miss // Rothstein Table 1, Col 6 spec: use scores for all students, but exclude teachers with missing VA
esttab _all, mtitles title("Missing Data Imputation Simulation Results") se not
}
else {
 reg d.score d.va_meas
 eststo `modname'_ideal
 reg d.score_miss d.va_miss
 eststo `modname'_miss
 reg d.score d.va_impute
 eststo `modname'_imp0
 }
*log close
end


*CFR model
 onesim
*With lower ICC
 onesim, icc(0.2) modname(lowICC)
*With EB shrinkage based on 4 years
 onesim, eb nyears(4) reliability(0.4) modname(fouryrs) 
*With both 
 onesim, eb nyears(4) reliability(0.4) modname(fouryrs_lowICC) icc(0.2)
*With EB shrinkage but no ICC.
 onesim, eb nyears(4) reliability(0.4) modname(fouryrs_noICC) icc(0)

 esttab Full_Sample No_Missing Impute_0s, se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("CFR parameters")
 esttab lowICC_ideal lowICC_miss lowICC_imp0, se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("Lower ICC (0.2)")
 esttab fouryrs_ideal fouryrs_miss fouryrs_imp0, se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast")
 esttab fouryrs_lowICC_ideal fouryrs_lowICC_miss fouryrs_lowICC_imp0, se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast, and ICC = 0.2")
 esttab fouryrs_noICC_ideal fouryrs_noICC_miss fouryrs_noICC_imp0, se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast, and ICC = 0")
 
 esttab Full_Sample No_Missing Impute_0s using `dofile'.txt, replace se nocons nogaps nostar ///
        mtitles(FullSamp ExcludeMiss Impute0) title("CFR parameters")
 esttab lowICC_ideal lowICC_miss lowICC_imp0 using `dofile'.txt, se nocons nogaps nostar append ///
        mtitles(FullSamp ExcludeMiss Impute0) title("Lower ICC (0.2)")
 esttab fouryrs_ideal fouryrs_miss fouryrs_imp0 using `dofile'.txt, se nocons nogaps nostar append ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast")
 esttab fouryrs_lowICC_ideal fouryrs_lowICC_miss fouryrs_lowICC_imp0 using `dofile'.txt, se nocons nogaps append ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast, and ICC = 0.2")
 esttab fouryrs_noICC_ideal fouryrs_noICC_miss fouryrs_noICC_imp0 using `dofile'.txt, se nocons nogaps append ///
        mtitles(FullSamp ExcludeMiss Impute0) title("With shrinkage & forecast, and ICC = 0")

 esttab Full_Sample No_Missing Impute_0s ///
        lowICC_ideal lowICC_miss lowICC_imp0 ///
        fouryrs_ideal fouryrs_miss fouryrs_imp0 ///
        fouryrs_lowICC_ideal fouryrs_lowICC_miss fouryrs_lowICC_imp0 ///
        fouryrs_noICC_ideal fouryrs_noICC_miss fouryrs_noICC_imp0 ///
        using `dofile'.csv, se nocons nogaps nostar  replace 

if `doasproject'==1 {
	 project, creates("`dofile'.txt")
	 project, creates("`dofile'.csv")
}
	